#!/usr/bin/env python3
# coding=utf-8
# author: @netmanchris
# -*- coding: utf-8 -*-
"""
This module contains functions for working with the netassets capabilities
of the HPE IMC NMS platform using the RESTful API

"""

# This section imports required libraries
import json

import requests

from pyhpeimc.auth import HEADERS

from pyhpeimc.plat.device import get_dev_details


def get_dev_asset_details(ipaddress, auth, url):
    """Takes in ipaddress as input to fetch device assett details from HP IMC RESTFUL API

    :param ipaddress: IP address of the device you wish to gather the asset details

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: object of type list containing the device asset details, with each asset contained
    in a dictionary

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.netassets import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> single_asset = get_dev_asset_details('10.101.0.1', auth.creds, auth.url)

    >>> assert type(single_asset) is list

    >>> assert 'name' in single_asset[0]

    """
    ipaddress = get_dev_details(ipaddress, auth, url)
    if isinstance(ipaddress, dict):
        ipaddress = ipaddress['ip']
    else:
        print("Asset Doesn't Exist")
        return 403
    f_url = url + "/imcrs/netasset/asset?assetDevice.ip=" + str(ipaddress)
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            dev_asset_info = (json.loads(response.text))
            if len(dev_asset_info) > 0:
                dev_asset_info = dev_asset_info['netAsset']
            if isinstance(dev_asset_info, dict):
                dev_asset_info = [dev_asset_info]
            if isinstance(dev_asset_info, list):
                dev_asset_info[:] = [dev for dev in dev_asset_info if dev.get('deviceIp') ==
                                     ipaddress]
            return dev_asset_info
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' get_dev_asset_details: An Error has occured'


def get_dev_asset_details_all(auth, url):
    """Takes no input to fetch device assett details from HP IMC RESTFUL API

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: list of dictionatires containing the device asset details

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.netassets import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> all_assets = get_dev_asset_details_all( auth.creds, auth.url)

    >>> assert type(all_assets) is list

    >>> assert 'asset' in all_assets[0]

    """
    f_url = url + "/imcrs/netasset/asset?start=0&size=15000"
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            dev_asset_info = (json.loads(response.text))['netAsset']
            return dev_asset_info
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' get_dev_asset_details: An Error has occured'



#!/usr/bin/env python3
# coding=utf-8
# author: @netmanchris
# -*- coding: utf-8 -*-
"""
This module contains functions for working with the groups capabilities
of the HPE IMC NMS platform using the RESTful API

"""

import json

import requests

from pyhpeimc.auth import HEADERS




# This section deals with HPE IMC Custom View functions


def get_custom_views(auth, url, name=None):
    """
    function requires no input and returns a list of dictionaries of custom views from an HPE
    IMC. Optional name argument will return only the specified view.
    :param name: str containing the name of the desired custom view

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param name: (optional) str of name of specific custom view

    :return: list of dictionaties containing attributes of the custom views

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.groups import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> all_views = get_custom_views(auth.creds, auth.url)

    >>> assert type(all_views) is list

    >>> assert 'name' in all_views[0]

    >>> non_existant_view = get_custom_views(auth.creds, auth.url, name = '''Doesn't Exist''')

    >>> assert non_existant_view is None


    """
    get_custom_view_url = None
    if name is None:
        get_custom_view_url = '/imcrs/plat/res/view/custom?resPrivilegeFilter=false&desc=false' \
                               '&total=false'
    elif name is not None:
        get_custom_view_url = '/imcrs/plat/res/view/custom?resPrivilegeFilter=false&name=' + \
                               name + '&desc=false&total=false'
    f_url = url + get_custom_view_url
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            custom_view_list = (json.loads(response.text))
            if 'customView' in custom_view_list:
                custom_view_list = custom_view_list['customView']
                if isinstance(custom_view_list, dict):
                    custom_view_list = [custom_view_list]
                    return custom_view_list
                else:
                    return custom_view_list
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' get_custom_views: An Error has occured'


def get_custom_view_details(name, auth, url):
    """
    function requires no input and returns a list of dictionaries of custom views from an HPE
    IMC. Optional name argument will return only the specified view.
    :param name: str containing the name of the desired custom view

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param name: (optional) str of name of specific custom view

    :return: list of dictionaties containing attributes of the custom views

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.groups import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> view_details = get_custom_view_details('My Network View', auth.creds, auth.url)

    >>> assert type(view_details) is list

    >>> assert 'label' in view_details[0]

    """
    view_id = get_custom_views(auth, url, name=name)
    if view_id is None:
        return view_id
    view_id = get_custom_views(auth, url, name=name)[0]['symbolId']
    get_custom_view_details_url = '/imcrs/plat/res/view/custom/' + str(view_id)
    f_url = url + get_custom_view_details_url
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            current_devices = (json.loads(response.text))
            if 'device' in current_devices:
                if isinstance(current_devices['device'], dict):
                    return [current_devices['device']]
                else:
                    return current_devices['device']
            else:
                return []
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' get_custom_views: An Error has occured'


def create_custom_views(auth, url, name=None, upperview=None):
    """
    function takes no input and issues a RESTFUL call to get a list of custom views from HPE IMC.
    Optional Name input will return only the specified view.

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param name: string containg the name of the desired custom view

    :param upperview: str contraining the name of the desired parent custom view

    :return: str of creation results ( "view " + name + "created successfully"

    :rtype: str

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.groups import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    #Create L1 custom view
    >>> create_custom_views(auth.creds, auth.url, name='L1 View')
    'View L1 View created successfully'

    >>> view_1 =get_custom_views( auth.creds, auth.url, name = 'L1 View')

    >>> assert type(view_1) is list

    >>> assert view_1[0]['name'] == 'L1 View'

    #Create Nested custome view
    >>> create_custom_views(auth.creds, auth.url, name='L2 View', upperview='L1 View')
    'View L2 View created successfully'

    >>> view_2 = get_custom_views( auth.creds, auth.url, name = 'L2 View')

    >>> assert type(view_2) is list

    >>> assert view_2[0]['name'] == 'L2 View'

    """
    create_custom_views_url = '/imcrs/plat/res/view/custom?resPrivilegeFilter=false&desc=false' \
                               '&total=false'
    f_url = url + create_custom_views_url
    if upperview is None:
        payload = '''{ "name": "''' + name + '''",
         "upLevelSymbolId" : ""}'''
    else:
        parentviewid = get_custom_views(auth, url, upperview)[0]['symbolId']
        payload = '''{ "name": "''' + name + '''",
        "upLevelSymbolId" : "''' + str(parentviewid) + '''"}'''
    response = requests.post(f_url, data=payload, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 201:
            print('View ' + name + ' created successfully')
            return response.status_code
        elif response.status_code == 409:
            print("View " + name + " already exists")
            return response.status_code
        else:
            return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' get_custom_views: An Error has occured'




def add_devs_custom_views(custom_view_name, dev_list, auth, url):
    """
    function takes a list of devIDs from devices discovered in the HPE IMC platform and issues a
    RESTFUL call to add the list of devices to a specific custom views from HPE IMC.

    :param custom_view_name: str of the target custom view name

    :param dev_list: list containing the devID of all devices to be contained in this custom view.

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: str of creation results ( "view " + name + "created successfully"

    :rtype: str

    >>> from pyhpeimc.auth import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    """
    view_id = get_custom_views(auth, url, name=custom_view_name)
    if view_id is None:
        print("View " + custom_view_name + " doesn't exist")
        return view_id
    view_id = get_custom_views(auth, url, name=custom_view_name)[0]['symbolId']
    add_devs_custom_views_url = '/imcrs/plat/res/view/custom/' + str(view_id)
    device_list = []
    for dev in dev_list:
        new_dev = {"id": dev}
        device_list.append(new_dev)
    payload = '''{"device" : ''' + json.dumps(device_list) + '''}'''
    print(payload)
    f_url = url + add_devs_custom_views_url
    response = requests.put(f_url, data=payload, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 204:
            print('View ' + custom_view_name + ' : Devices Successfully Added')
            return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' get_custom_views: An Error has occured'


def delete_custom_view(auth, url, name):
    """
    function takes input of auth, url, and name and issues a RESTFUL call to delete a specific
    of custom views from HPE
    IMC.
    :param name: string containg the name of the desired custom view

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: str of creation results ( "view " + name + "created successfully"

    :rtype: str

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.groups import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> delete_custom_view(auth.creds, auth.url, name = "L1 View")
    'View L1 View deleted successfully'

    >>> view_1 =get_custom_views( auth.creds, auth.url, name = 'L1 View')

    >>> assert view_1 is None

    >>> delete_custom_view(auth.creds, auth.url, name = "L2 View")
    'View L2 View deleted successfully'

    >>> view_2 =get_custom_views( auth.creds, auth.url, name = 'L2 View')

    >>> assert view_2 is None

    """
    view_id = get_custom_views(auth, url, name)
    if view_id is None:
        print("View " + name + " doesn't exists")
        return view_id
    view_id = get_custom_views(auth, url, name)[0]['symbolId']
    delete_custom_view_url = '/imcrs/plat/res/view/custom/' + str(view_id)
    f_url = url + delete_custom_view_url
    response = requests.delete(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 204:
            print('View ' + name + ' deleted successfully')
            return response.status_code
        else:
            return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' delete_custom_view: An Error has occured'



#!/usr/bin/env python3
# coding=utf-8
# author: @netmanchris
# -*- coding: utf-8 -*-
"""
This module contains functions for working with the system configuration
capabilities of the HPE IMC NMS platform using the RESTful API

"""


# This section imports required libraries
import json

import requests

from pyhpeimc.auth import HEADERS


# This section contains functions which operate at the system level


def get_system_vendors(auth, url):
    """Takes string no input to issue RESTUL call to HP IMC\n

      :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

      :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

      :return: list of dictionaries where each dictionary represents a single vendor

      :rtype: list

      >>> from pyhpeimc.auth import *

      >>> from pyhpeimc.plat.system import *

      >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

      >>> vendors = get_system_vendors(auth.creds, auth.url)

      >>> assert type(vendors) is list

      >>> assert 'name' in vendors[0]


    """
    f_url = url + '/imcrs/plat/res/vendor?start=0&size=10000&orderBy=id&desc=false&total=false'
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            system_vendors = (json.loads(response.text))
            return system_vendors['deviceVendor']
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_dev_details: An Error has occured"


def get_system_category(auth, url):
    """Takes string no input to issue RESTUL call to HP IMC\n

      :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

      :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

      :return: list of dictionaries where each dictionary represents a single device category

      :rtype: list

      >>> from pyhpeimc.auth import *

      >>> from pyhpeimc.plat.device import *

      >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

      >>> categories = get_system_category(auth.creds, auth.url)

      >>> assert type(categories) is list

      >>> assert 'name' in categories[0]


    """
    f_url = url + '/imcrs/plat/res/category?start=0&size=10000&orderBy=id&desc=false&total=false'
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            system_category = (json.loads(response.text))
            return system_category['deviceCategory']
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_dev_details: An Error has occured"


def get_system_device_models(auth, url):
    """Takes string no input to issue RESTUL call to HP IMC

      :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

      :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

      :return: list of dictionaries where each dictionary represents a single device model

      :rtype: list


      >>> from pyhpeimc.auth import *

      >>> from pyhpeimc.plat.device import *

      >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

      >>> device_models = get_system_device_models(auth.creds, auth.url)

      >>> assert type(device_models) is list

      >>> assert 'virtualDeviceName' in device_models[0]

    """
    f_url = url + '/imcrs/plat/res/model?start=0&size=10000&orderBy=id&desc=false&total=false'
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            system_device_model = (json.loads(response.text))
            return system_device_model['deviceModel']
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_dev_details: An Error has occured"


def get_system_series(auth, url):
    """Takes no input to issue RESTUL call to HP IMC

      :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

      :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

      :return: list of dictionaries where each dictionary represents a single device series

      :rtype: list

      >>> from pyhpeimc.auth import *

      >>> from pyhpeimc.plat.device import *

      >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

      >>> series = get_system_series(auth.creds, auth.url)

      >>> assert type(series) is list

      >>> assert 'name' in series[0]

    """
    f_url = url + '/imcrs/plat/res/series?managedOnly=false&start=0&size=10000&orderBy=id&desc' \
                   '=false&total=false'
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            system_series = (json.loads(response.text))
            return system_series['deviceSeries']
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_dev_series: An Error has occured"

# This section deals with manipulating system authentication credentials

# Telnet Templates

def create_telnet_template(auth, url, telnet_template ):
    """
    Function takes input of a dictionry containing the required key/value pair for the creation
    of a telnet template.

    :param auth:
    :param url:
    :param telnet_template: dictionary of valid JSON which complains to API schema
    :return: int value of HTTP response code 201 for proper creation or 404 for failed creation
    :rtype int

    Sample of proper KV pairs. Please see documentation for valid values for different fields.

    telnet_template = {"type": "0",
    "name": "User_with_Enable",
    "authType": "4",
    "userName": "admin",
    "userPassword": "password",
    "superPassword": "password",
    "authTypeStr": "Username + Password + Super/Manager Password",
    "timeout": "4",
    "retries": "1",
    "port": "23",
    "version": "1",
    "creator": "admin",
    "accessType": "1",
    "operatorGroupStr": ""}
    """
    f_url = url + "/imcrs/plat/res/telnet/add"
    response = requests.post(f_url, data = json.dumps(telnet_template), auth=auth, headers=HEADERS)
    try:
        return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " create_telnet_template: An Error has occured"

def get_telnet_template(auth, url, template_name=None):
    """
    Takes no input, or template_name as input to issue RESTUL call to HP IMC

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param template_name: str value of template name

    :return list object containing one or more dictionaries where each dictionary represents one
    telnet template

    :rtype list

    """
    f_url = url + "/imcrs/plat/res/telnet?start=0&size=10000&desc=false&total=false"
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            telnet_templates = (json.loads(response.text))
            template = None
            if type(telnet_templates['telnetParamTemplate']) is dict:
                my_templates = [telnet_templates['telnetParamTemplate']]
                telnet_templates['telnetParamTemplate'] = my_templates
            if template_name is None:
                return telnet_templates['telnetParamTemplate']
            elif template_name is not None:
                for telnet_template in telnet_templates['telnetParamTemplate']:

                    if telnet_template['name'] == template_name:
                        template = [telnet_template]
                print (type(template))
                if template == None:
                    return 404
                else:
                    return template
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_telnet_templates: An Error has occured"


def modify_telnet_template(auth, url, telnet_template, template_name= None, template_id = None):
    """
    Function takes input of a dictionry containing the required key/value pair for the modification
    of a telnet template.

    :param auth:
    :param url:
    :param telnet_template: Human readable label which is the name of the specific telnet template
    :param template_id Internal IMC number which designates the specific telnet template
    :return: int value of HTTP response code 201 for proper creation or 404 for failed creation
    :rtype int

    Sample of proper KV pairs. Please see documentation for valid values for different fields.

    telnet_template = {"type": "0",
    "name": "User_with_Enable",
    "authType": "4",
    "userName": "newadmin",
    "userPassword": "newpassword",
    "superPassword": "newpassword",
    "authTypeStr": "Username + Password + Super/Manager Password",
    "timeout": "4",
    "retries": "1",
    "port": "23",
    "version": "1",
    "creator": "admin",
    "accessType": "1",
    "operatorGroupStr": ""}
    """
    if template_name is None:
        template_name = telnet_template['name']
    if template_id is None:
        telnet_templates = get_telnet_template(auth, url)
        template_id = None
        for template in telnet_templates:
            if template['name'] == template_name:
                template_id = template['id']
    f_url = url + "/imcrs/plat/res/telnet/"+str(template_id)+"/update"
    response = requests.put(f_url, data = json.dumps(telnet_template), auth=auth, headers=HEADERS)
    try:
        return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " modify_telnet_template: An Error has occured"


def delete_telnet_template(auth, url, template_name= None, template_id= None):
    """
    Takes template_name as input to issue RESTUL call to HP IMC which will delete the specific
    telnet template from the IMC system

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param template_name: str value of template name

    :param template_id: str value template template_id value

    :return: int HTTP response code

    :rtype  int
    """
    try:
        if template_id is None:
            telnet_templates = get_telnet_template(auth, url)
            if template_name is None:
                template_name = telnet_template['name']
            template_id = None
            for template in telnet_templates:
                if template['name'] == template_name:
                    template_id = template['id']
        f_url = url + "/imcrs/plat/res/telnet/%s/delete" % template_id
        response = requests.delete(f_url, auth=auth, headers=HEADERS)
        return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " delete_telnet_template: An Error has occured"


# SSH Templates

def create_ssh_template(auth, url, ssh_template ):
    """
    Function takes input of a dictionry containing the required key/value pair for the creation
    of a ssh template.

    :param auth:
    :param url:
    :param ssh: dictionary of valid JSON which complains to API schema
    :return: int value of HTTP response code 201 for proper creation or 404 for failed creation
    :rtype int

    Sample of proper KV pairs. Please see documentation for valid values for different fields.

    ssh_template = {
    "type": "0",
    "name": "ssh_admin_template",
    "authType": "3",
    "authTypeStr": "Password + Super Password",
    "userName": "admin",
    "password": "password",
    "superPassword": "password",
    "port": "22",
    "timeout": "10",
    "retries": "3",
    "keyFileName": "",
    "keyPhrase": ""
    }
    """
    f_url = url + "/imcrs/plat/res/ssh/add"
    response = requests.post(f_url, data = json.dumps(ssh_template), auth=auth, headers=HEADERS)
    try:
        return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " create_ssh_template: An Error has occured"

def get_ssh_template(auth, url, template_name=None):
    """
    Takes no input, or template_name as input to issue RESTUL call to HP IMC

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param template_name: str value of template name

    :return list object containing one or more dictionaries where each dictionary represents one
    ssh template

    :rtype list

    """
    f_url = url + "/imcrs/plat/res/ssh?start=0&size=10000&desc=false&total=false"
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            ssh_templates = (json.loads(response.text))
            template = None
            if type(ssh_templates['sshParamTemplate']) is dict:
                my_templates = [ssh_templates['sshParamTemplate']]
                ssh_templates['sshParamTemplate'] = my_templates
            if template_name is None:
                return ssh_templates['sshParamTemplate']
            elif template_name is not None:
                for ssh_template in ssh_templates['sshParamTemplate']:
                    if ssh_template['name'] == template_name:
                        template = [ssh_template]
                print (type(template))
                if template == None:
                    return 404
                else:
                    return template
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_ssh_templates: An Error has occured"


def modify_ssh_template(auth, url, ssh_template, template_name= None, template_id = None):
    """
    Function takes input of a dictionry containing the required key/value pair for the modification
    of a ssh template.

    :param auth:
    :param url:
    :param ssh_template: Human readable label which is the name of the specific ssh template
    :param template_id Internal IMC number which designates the specific ssh template
    :return: int value of HTTP response code 201 for proper creation or 404 for failed creation
    :rtype int

    Sample of proper KV pairs. Please see documentation for valid values for different fields.

    ssh_template = {
    "type": "0",
    "name": "ssh_admin_template",
    "authType": "3",
    "authTypeStr": "Password + Super Password",
    "userName": "newadmin",
    "password": "password",
    "superPassword": "password",
    "port": "22",
    "timeout": "10",
    "retries": "3",
    "keyFileName": "",
    "keyPhrase": ""
    }
    """
    if template_name is None:
        template_name = ssh_template['name']
    if template_id is None:
        ssh_templates = get_ssh_template(auth, url)
        template_id = None
        for template in ssh_templates:
            if template['name'] == template_name:
                template_id = template['id']
    f_url = url + "/imcrs/plat/res/ssh/"+str(template_id)+"/update"
    response = requests.put(f_url, data = json.dumps(ssh_template), auth=auth, headers=HEADERS)
    try:
        return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " modify_ssh_template: An Error has occured"


def delete_ssh_template(auth, url, template_name= None, template_id= None):
    """
    Takes template_name as input to issue RESTUL call to HP IMC which will delete the specific
    ssh template from the IMC system

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param template_name: str value of template name

    :param template_id: str value template template_id value

    :return: int HTTP response code

    :rtype  int
    """
    try:
        if template_id is None:
            ssh_templates = get_ssh_template(auth, url)
            if template_name is None:
                template_name = ssh_template['name']
            template_id = None
            for template in ssh_templates:
                if template['name'] == template_name:
                    template_id = template['id']
        f_url = url + "/imcrs/plat/res/ssh/%s/delete" % template_id
        response = requests.delete(f_url, auth=auth, headers=HEADERS)
        return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " delete_ssh_template: An Error has occured"












# snmp Templates

def create_snmp_template(auth, url, snmp_template ):
    """
    Function takes input of a dictionry containing the required key/value pair for the creation
    of a snmp template.

    :param auth:
    :param url:
    :param ssh: dictionary of valid JSON which complains to API schema
    :return: int value of HTTP response code 201 for proper creation or 404 for failed creation
    :rtype int

    Sample of proper KV pairs. Please see documentation for valid values for different fields.

    snmp_template =  {
      "version": "2",
      "name": "new_snmp_template",
      "type": "0",
      "paraType": "SNMPv2c",
      "roCommunity": "public",
      "rwCommunity": "private",
      "timeout": "4",
      "retries": "3",
      "contextName": "",
      "securityName": " ",
      "securityMode": "1",
      "authScheme": "0",
      "authPassword": "",
      "privScheme": "0",
      "privPassword": "",
      "snmpPort": "161",
      "isAutodiscoverTemp": "1",
      "creator": "admin",
      "accessType": "1",
      "operatorGroupStr": ""
      }
    """
    f_url = url + "/imcrs/plat/res/snmp/add"
    response = requests.post(f_url, data = json.dumps(snmp_template), auth=auth, headers=HEADERS)
    try:
        return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " create_snmp_template: An Error has occured"

def get_snmp_templates(auth, url, template_name=None):
    """
    Takes no input, or template_name as input to issue RESTUL call to HP IMC

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param template_name: str value of template name

    :return list object containing one or more dictionaries where each dictionary represents one
    snmp template

    :rtype list

    """
    f_url = url + "/imcrs/plat/res/snmp?start=0&size=10000&desc=false&total=false"
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            snmp_templates = (json.loads(response.text))
            template = None
            if type(snmp_templates['snmpParamTemplate']) is dict:
                my_templates = [snmp_templates['snmpParamTemplate']]
                snmp_templates['snmpParamTemplate'] = my_templates
            if template_name is None:
                return snmp_templates['snmpParamTemplate']
            elif template_name is not None:
                for snmp_template in snmp_templates['snmpParamTemplate']:
                    if snmp_template['name'] == template_name:
                        template = [snmp_template]
                if template == None:
                    return 404
                else:
                    return template
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_snmp_templates: An Error has occured"


def modify_snmp_template(auth, url, snmp_template, template_name= None, template_id = None):
    """
    Function takes input of a dictionry containing the required key/value pair for the modification
    of a snmp template.

    :param auth:
    :param url:
    :param ssh_template: Human readable label which is the name of the specific ssh template
    :param template_id Internal IMC number which designates the specific ssh template
    :return: int value of HTTP response code 201 for proper creation or 404 for failed creation
    :rtype int

    Sample of proper KV pairs. Please see documentation for valid values for different fields.

    snmp_template =  {
      "version": "2",
      "name": "new_snmp_template",
      "type": "0",
      "paraType": "SNMPv2c",
      "roCommunity": "newpublic",
      "rwCommunity": "newprivate",
      "timeout": "4",
      "retries": "4",
      "contextName": "",
      "securityName": " ",
      "securityMode": "1",
      "authScheme": "0",
      "authPassword": "",
      "privScheme": "0",
      "privPassword": "",
      "snmpPort": "161",
      "isAutodiscoverTemp": "1",
      "creator": "admin",
      "accessType": "1",
      "operatorGroupStr": ""
      }
    """
    if template_name is None:
        template_name = snmp_template['name']
    if template_id is None:
        snmp_templates = get_snmp_templates(auth, url)
        template_id = None
        for template in snmp_templates:
            if template['name'] == template_name:
                template_id = template['id']
    f_url = url + "/imcrs/plat/res/snmp/"+str(template_id)+"/update"
    response = requests.put(f_url, data = json.dumps(snmp_template), auth=auth, headers=HEADERS)
    try:
        return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " modify_snmp_template: An Error has occured"


def delete_snmp_template(auth, url, template_name= None, template_id= None):
    """
    Takes template_name as input to issue RESTUL call to HP IMC which will delete the specific
    snmp template from the IMC system

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param template_name: str value of template name

    :param template_id: str value template template_id value

    :return: int HTTP response code

    :rtype  int
    """
    try:
        if template_id is None:
            snmp_templates = get_snmp_templates(auth, url)
            if template_name is None:
                template_name = snmp_template['name']
            template_id = None
            for template in snmp_templates:
                if template['name'] == template_name:
                    template_id = template['id']
        f_url = url + "/imcrs/plat/res/snmp/%s/delete" % template_id
        response = requests.delete(f_url, auth=auth, headers=HEADERS)
        return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " delete_snmp_template: An Error has occured"




#!/usr/bin/env python3
# coding=utf-8
# author: @netmanchris
# -*- coding: utf-8 -*-
"""
This module contains functions for working with the terminal access
capabilities, including IP Address management of the HPE IMC NMS
platform using the RESTful API

"""

# This section imports required libraries
import ipaddress
import json

import requests

from pyhpeimc.auth import HEADERS
from pyhpeimc.plat.device import get_dev_details

#pylint: disable=R0913

def get_real_time_locate(host_ipaddress, auth, url):
    """
    function takes the ipAddress of a specific host and issues a RESTFUL call to get the device and
    interface that the target host is currently connected to. Note: Although intended to return a
    single location, Multiple locations may be returned for a single host due to a partially
    discovered network or misconfigured environment.

    :param host_ipaddress: str value valid IPv4 IP address

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: list of dictionaries where each element of the list represents the location of the
    target host

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> found_device = get_real_time_locate('10.101.0.51', auth.creds, auth.url)

    >>> assert type(found_device) is list

    >>> assert 'deviceId' in found_device[0]

    >>> assert 'deviceId' in found_device[0]

    >>> assert 'deviceId' in found_device[0]

    >>> assert 'deviceId' in found_device[0]

    >>> no_device = get_real_time_locate('192.168.254.254', auth.creds, auth.url)

    >>> assert type(no_device) is dict

    >>> assert len(no_device) == 0

    """
    f_url = url + "/imcrs/res/access/realtimeLocate?type=2&value=" + str(host_ipaddress)  + \
            "&total=false"
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            response = json.loads(response.text)
            if 'realtimeLocation' in response:
                real_time_locate = response['realtimeLocation']
                if isinstance(real_time_locate, dict):
                    real_time_locate = [real_time_locate]
                    return real_time_locate
                else:
                    return json.loads(response)['realtimeLocation']
            else:
                print("Host not found")
                return 403
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_real_time_locate: An Error has occured"


def get_ip_mac_arp_list(auth, url, devid=None, devip=None):
    """
    function takes devid of specific device and issues a RESTFUL call to get the IP/MAC/ARP list
    from the target device.

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param devid: int or str value of the target device.

    :param devip: str of ipv4 address of the target device

    :return: list of dictionaries containing the IP/MAC/ARP list of the target device.

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> ip_mac_list = get_ip_mac_arp_list( auth.creds, auth.url, devid='10')

    >>> ip_mac_list = get_ip_mac_arp_list( auth.creds, auth.url, devip='10.101.0.221')

    >>> assert type(ip_mac_list) is list

    >>> assert 'deviceId' in ip_mac_list[0]

    """
    if devip is not None:
        dev_details = get_dev_details(devip, auth, url)
        if isinstance(dev_details, str):
            print("Device not found")
            return 403
        else:
            devid = get_dev_details(devip, auth, url)['id']
    f_url = url + "/imcrs/res/access/ipMacArp/" + str(devid)
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            ipmacarplist = (json.loads(response.text))
            if 'ipMacArp' in ipmacarplist:
                return ipmacarplist['ipMacArp']
            else:
                return ['this function is unsupported']
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_ip_mac_arp_list: An Error has occured"


# this section deals with the IP Address Manager functions with terminal access of HPE IMC Base
# platform


# Following functions deal with IP scopes
def get_ip_scope(auth, url, scopeid=None, ):
    """
    function requires no inputs and returns all IP address scopes currently configured on the HPE
    IMC server. If the optional scopeid parameter is included, this will automatically return
    only the desired scope id.

    :param scopeid: integer of the desired scope id ( optional )

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: list of dictionary objects where each element of the list represents one IP scope

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> ip_scope_list = get_ip_scope(auth.creds, auth.url)

    >>> assert type(ip_scope_list) is list

    >>> assert 'ip' in ip_scope_list[0]

    """
    if scopeid is None:
        get_ip_scope_url = "/imcrs/res/access/assignedIpScope"
    else:
        get_ip_scope_url = "/imcrs/res/access/assignedIpScope/ip?ipScopeId=" + str(scopeid)

    f_url = url + get_ip_scope_url
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            ipscopelist = (json.loads(response.text))['assignedIpScope']
            if isinstance(ipscopelist, list):
                return ipscopelist
            elif isinstance(ipscopelist, dict):
                return [ipscopelist]
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_ip_scope: An Error has occured"


def get_ip_scope_detail(auth, url, scopeid=None, network_address=None):
    """
    function requires no inputs and returns all IP address scopes currently configured on the HPE
    IMC server. If the optional scopeId parameter is included, this will automatically return
    only the desired scope id.

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param scopeid: integer of the desired scope id ( optional )

    :param network_address: ipv4 network address + subnet bits of target scope

    :return: dictionary, may containing multiple entries if sub-scopes have been created

    :rtype: dict

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> ip_scope_detail = get_ip_scope_detail( auth.creds, auth.url, scopeId = '45')

    >>> assert type(ip_scope_detail) is dict

    >>> assert 'startIp' in ip_scope_detail

    """
    if network_address is not None:
        scopeid = get_scope_id(network_address, auth, url)
    f_url = url + "/imcrs/res/access/assignedIpScope/" + str(scopeid)
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            ipscopelist = (json.loads(response.text))
            return ipscopelist
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_ip_scope: An Error has occured"


def add_ip_scope(name, description, auth, url, startip=None, endip=None, network_address=None):
    """
    Function takes input of four strings Start Ip, endIp, name, and description to add new Ip Scope
    to terminal access in the HPE IMC base platform

    :param name: str Name of the owner of this IP scope  ex. 'admin'

    :param description: str description of the Ip scope

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param startip: str Start of IP address scope ex. '10.101.0.1'

    :param endip: str End of IP address scope ex. '10.101.0.254'

    :param network_address: ipv4 network address + subnet bits of target scope

    :return: 200 if successfull

    :rtype:

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> delete_ip_scope('10.50.0.0/24', auth.creds, auth.url)
    <Response [204]>

    >>> new_scope = add_ip_scope('10.50.0.1', '10.50.0.254', 'cyoung', 'test group', auth.creds, auth.url)

    >>> assert type(new_scope) is int

    >>> assert new_scope == 200

    >>> existing_scope = add_ip_scope('10.50.0.1', '10.50.0.254', 'cyoung', 'test group', auth.creds, auth.url)

    >>> assert type(existing_scope) is int

    >>> assert existing_scope == 409


    """
    if network_address is not None:
        nw_address = ipaddress.IPv4Network(network_address)
        startip = nw_address[1]
        endip = nw_address[-2]
    f_url = url + "/imcrs/res/access/assignedIpScope"
    payload = ('''{  "startIp": "%s", "endIp": "%s","name": "%s","description": "%s" }'''
               % (str(startip), str(endip), str(name), str(description)))
    response = requests.post(f_url, auth=auth, headers=HEADERS, data=payload)
    try:
        if response.status_code == 200:
            # print("IP Scope Successfully Created")
            return response.status_code
        elif response.status_code == 409:
            # print ("IP Scope Already Exists")
            return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " add_ip_scope: An Error has occured"


def add_child_ip_scope(name, description, auth, url, startip=None, endip=None, parent_scopeid=None,
                       network_address=None, parent_network_address=None):
    """
    Function takes input of four strings Start Ip, endIp, name, and description to add new Ip Scope
    to terminal access in the HPE IMC base platform

    :param name: str Name of the owner of this IP scope  ex. 'admin'

    :param description: str description of the Ip scope

    :param parent_scopeid: str or int of the scopeid of the target scope

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param startip: str Start of IP address scope ex. '10.101.0.1'

    :param endip: str End of IP address scope ex. '10.101.0.254'

    :param network_address: ipv4 network address + subnet bits of target scope

    :param parent_network_address: ipv4 network address + subnet bits of target parent scope

    :return: 200

    :rtype:

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> add_child_ip_scope('10.50.0.1', '10.50.0.126', 'cyoung', 'test sub scope', '175', auth.creds, auth.url)

    """
    if parent_network_address is not None:
        parent_scopeid = get_scope_id(parent_network_address, auth, url)
    if network_address is not None:
        nw_address = ipaddress.IPv4Network(network_address)
        startip = nw_address[1]
        endip = nw_address[-2]
    f_url = url + "/imcrs/res/access/assignedIpScope/" + str(parent_scopeid)
    payload = ('''{ "startIp": "%s", "endIp": "%s","name": "%s","description": "%s",
                    "parentId" : "%s"}'''
               % (str(startip), str(endip), str(name), str(description), str(parent_scopeid)))
    response = requests.post(f_url, auth=auth, headers=HEADERS, data=payload)
    try:
        if response.status_code == 200:
            # print("IP Scope Successfully Created")
            return response.status_code
        elif response.status_code == 409:
            # print ("Conflict with Current Scope")
            return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " add_ip_scope: An Error has occured"


def delete_ip_scope(network_address, auth, url):
    """
    Function to delete an entire IP segment from the IMC IP Address management under terminal access
    :param network_address
    :param auth
    :param url

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> new_scope = add_ip_scope('10.50.0.1', '10.50.0.254', 'cyoung', 'test group', auth.creds, auth.url)

    >>> delete_scope = delete_ip_scope('10.50.0.0/24', auth.creds, auth.url)


    """
    scope_id = get_scope_id(network_address, auth, url)
    if scope_id == "Scope Doesn't Exist":
        return scope_id
    f_url = url + '''/imcrs/res/access/assignedIpScope/''' + str(scope_id)
    response = requests.delete(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 204:
            # print("IP Segment Successfully Deleted")
            return 204
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " delete_ip_scope: An Error has occured"


# Following functions deal with hosts assigned to IP scopes

def add_scope_ip(hostipaddress, name, description, auth, url, scopeid=None, network_address=None):
    """
    Function to add new host IP address allocation to existing scope ID

    :param hostipaddress: ipv4 address of the target host to be added to the target scope

    :param name: name of the owner of this host

    :param description: Description of the host

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param scopeid: integer of the desired scope id ( optional )

    :param network_address: ipv4 network address + subnet bits of target scope

    :return:

    :rtype:

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> new_host = add_scope_ip('10.50.0.5', 'cyoung', 'New Test Host','175', auth.creds, auth.url)

    """
    if network_address is not None:
        scopeid = get_scope_id(network_address, auth, url)
        if scopeid == "Scope Doesn't Exist":
            return scopeid
    new_ip = {"ip": hostipaddress,
              "name": name,
              "description": description}
    f_url = url + '/imcrs/res/access/assignedIpScope/ip?ipScopeId=' + str(scopeid)
    payload = json.dumps(new_ip)
    response = requests.post(f_url, auth=auth, headers=HEADERS, data=payload)
    try:
        if response.status_code == 200:
            # print("IP Host Successfully Created")
            return response.status_code
        elif response.status_code == 409:
            # print("IP Host Already Exists")
            return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " add_ip_scope: An Error has occured"


def remove_scope_ip(hostid, auth, url):
    """
    Function to add remove IP address allocation

    :param hostid: Host id of the host to be deleted

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: String of HTTP response code. Should be 204 is successfull

    :rtype: str

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> new_scope = add_ip_scope('10.50.0.1', '10.50.0.254', 'cyoung', 'test group', auth.creds, auth.url)

    >>> add_host_to_segment('10.50.0.5', 'cyoung', 'New Test Host', '10.50.0.0/24', auth.creds, auth.url)

    >>> host_id = get_host_id('10.50.0.5', '10.50.0.0/24', auth.creds, auth.url)

    >>> rem_host = remove_scope_ip(host_id, auth.creds, auth.url)

    >>> assert type(rem_host) is int

    >>> assert rem_host == 204

    """
    f_url = url + '/imcrs/res/access/assignedIpScope/ip/' + str(hostid)
    response = requests.delete(f_url, auth=auth, headers=HEADERS, )
    try:
        if response.status_code == 204:
            # print("Host Successfully Deleted")
            return response.status_code
        elif response.status_code == 409:
            # print("IP Scope Already Exists")
            return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " add_ip_scope: An Error has occured"


def get_ip_scope_hosts(auth, url, scopeid=None, network_address=None):
    """
    Function requires input of scope ID and returns list of allocated IP address for the
    specified scope

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param scopeid: Integer of the desired scope id

    :param network_address: ipv4 network address + subnet bits of target scope

    :return: list of dictionary objects where each element of the list represents a single host
    assigned to the IP scope

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> scope_id = get_scope_id('10.50.0.0/24', auth.creds, auth.url)

    >>> ip_scope_hosts = get_ip_scope_hosts(scope_id, auth.creds, auth.url)

    >>> assert type(ip_scope_hosts) is list

    >>> assert 'name' in ip_scope_hosts[0]

    >>> assert 'description' in ip_scope_hosts[0]

    >>> assert 'ip' in ip_scope_hosts[0]

    >>> assert 'id' in ip_scope_hosts[0]

    """
    if network_address is not None:
        scopeid = get_scope_id(network_address, auth, url)
        if scopeid == "Scope Doesn't Exist":
            return scopeid
    f_url = url + "/imcrs/res/access/assignedIpScope/ip?size=10000&ipScopeId=" + str(scopeid)
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            ipscopelist = (json.loads(response.text))
            if ipscopelist == {}:
                return [ipscopelist]
            else:
                ipscopelist = ipscopelist['assignedIpInfo']
            if isinstance(ipscopelist, dict):
                ipscope = [ipscopelist]
                return ipscope
            return ipscopelist
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_ip_scope: An Error has occured"


def add_host_to_segment(hostipaddress, name, description, network_address, auth, url):
    """
    Function to abstract existing add_scope_ip_function. Allows for use of network address rather
    than forcing human to learn the scope_id
    :param hostipaddress: str ipv4 address of target host to be added to target scope

    :param name: name of the owner of this host

    :param description: Description of the host

    :param network_address: ipv4 network address + subnet bits of target scope

    :param: network_address: network address of the target scope in format x.x.x.x/yy  where
    x.x.x.x  representents the network address and yy represents the length of the subnet mask.
    Example:  10.50.0.0 255.255.255.0 would be written as 10.50.0.0/24

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return:

    :rtype:

    """
    scope_id = get_scope_id(network_address, auth, url)
    add_scope_ip(hostipaddress, name, description, scope_id, auth, url)


def delete_host_from_segment(hostipaddress, networkaddress, auth, url):
    """
    :param hostipaddress: str ipv4 address of the target host to be deleted

    :param networkaddress: ipv4 network address + subnet bits of target scope

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: String of HTTP response code. Should be 204 is successfull

    :rtype: str

    """
    host_id = get_host_id(hostipaddress, networkaddress, auth, url)
    delete_host = remove_scope_ip(host_id, auth, url)
    return delete_host



# Following Section are Helper functions to help translate human readable IPv4 address to IMC
# internal keys for working with IPscopes and hosts


def get_scope_id(network_address, auth, url):
    """

    :param network_address: network address of the target scope in format x.x.x.x/yy  where
    x.x.x.x  representents the network address and yy represents the length of the subnet mask.
    Example:  10.50.0.0 255.255.255.0 would be written as 10.50.0.0/24

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: str object which contains the numerical ID of the target scope

    :rtype: str

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> new_scope = add_ip_scope('10.50.0.1', '10.50.0.254', 'cyoung', 'test group', auth.creds, auth.url)

    >>> scope_id = get_scope_id('10.50.0.0/24', auth.creds, auth.url)


    >>> assert type(scope_id) is str

    """
    netaddr = ipaddress.ip_network(network_address)
    scopes = get_ip_scope(auth, url)
    for parent_scope in scopes:
        if int(parent_scope['id']) > 0:
            if "assignedIpScope" in parent_scope:
                child_scope = parent_scope['assignedIpScope']
                if isinstance(child_scope, dict):
                    child_scope = [child_scope]
                for scope in child_scope:
                    if ipaddress.ip_address(scope['startIp']) == netaddr[1] \
                            and ipaddress.ip_address(scope['endIp']) == netaddr[-2]:
                        return scope['id']
            if ipaddress.ip_address(parent_scope['startIp']) == netaddr[1] and ipaddress.ip_address(
                    parent_scope['endIp']) == netaddr[-2]:
                return parent_scope['id']
    return "Scope Doesn't Exist"


def get_host_id(host_address, network_address, auth, url):
    """
    Function takes str of ipv4 host address and str of ipv4 networkaddress (CIDR format) with
    auth and url and sends restul call to HPE IMC NMS. Expected return is a str value which
    represents the numerical ID of the target scope

    :param host_address: str describing network address of the target scope in format x.x.x.x
    where x.x.x.x representents the full ipv4 address.  Example:  10.50.0.5 255.255.255.0 would
    be written as 10.50.0.5

    :param network_address: network address of the target scope in format x.x.x.x/yy  where
    x.x.x.x  representents the network address and yy represents the length of the subnet mask.
    Example:  10.50.0.0 255.255.255.0 would be written as 10.50.0.0/24

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: str object which contains the numerical ID of the target scope

    :rtype: str
    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.termaccess import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> new_scope = add_ip_scope('10.50.0.1', '10.50.0.254', 'cyoung', 'test group', auth.creds, auth.url)

    >>> add_host_to_segment('10.50.0.5', 'cyoung', 'New Test Host', '10.50.0.0/24', auth.creds, auth.url)

    >>> new_host_id = get_host_id('10.50.0.5', '10.50.0.0/24', auth.creds, auth.url)

    >>> assert type(new_host_id) is str

    """
    scope_id = get_scope_id(network_address, auth, url)
    if scope_id == "Scope Doesn't Exist":
        return scope_id
    all_scope_hosts = get_ip_scope_hosts(auth, url, scopeid=scope_id)
    if len(all_scope_hosts) == 0:
        return "Host Doesn't Exist"
    if 'ip' in all_scope_hosts[0]:
        for host in all_scope_hosts:
            if host['ip'] == host_address:
                return host['id']
    else:
        return "Host Doesn't Exist"
